@@ -20,23 +20,27 @@
 #' get_mass_props(mp_table, "C.1.2.2.3.1.2.3")
 #'
 get_mass_props <- function(df, id) {
-  poi_conv <- df_get_by_id(df, id, "POIconv")
+  row <- df_get_row_by_id(df, id)
+  poi_conv <- row$POIconv
   list(
-    mass = df_get_by_id(df, id, "mass"),
-    center_mass = sapply(c(x = "Cx", y = "Cy", z = "Cz"), FUN=function(p) df_get_by_id(df, id, p)),
+    mass = row$mass,
+
+    center_mass = c(x = row$Cx, y = row$Cy, z = row$Cz),
+
     inertia = {
       xyz <- list("x", "y", "z")
       it <- matrix(data = rep.int(0, 9), nrow = 3, dimnames = list(xyz, xyz))
-      it["x", "x"] <- df_get_by_id(df, id, "Ixx")
-      it["y", "y"] <- df_get_by_id(df, id, "Iyy")
-      it["z", "z"] <- df_get_by_id(df, id, "Izz")
+      it["x", "x"] <- row$Ixx
+      it["y", "y"] <- row$Iyy
+      it["z", "z"] <- row$Izz
       poi_factor <- if (poi_conv == '-') 1 else -1
-      it["x", "y"] <- it["y", "x"] <- poi_factor * df_get_by_id(df, id, "Ixy")
-      it["x", "z"] <- it["z", "x"] <- poi_factor * df_get_by_id(df, id, "Ixz")
-      it["y", "z"] <- it["z", "y"] <- poi_factor * df_get_by_id(df, id, "Iyz")
+      it["x", "y"] <- it["y", "x"] <- poi_factor * row$Ixy
+      it["x", "z"] <- it["z", "x"] <- poi_factor * row$Ixz
+      it["y", "z"] <- it["z", "y"] <- poi_factor * row$Iyz
       it
     },
-    point = df_get_by_id(df, id, "Ipoint")
+
+    point = row[["Ipoint"]]
   )
 }
 
@@ -60,18 +64,21 @@ get_mass_props <- function(df, id) {
 #' get_mass_props_unc(mp_table, "C.1.2.2.3.1.2.3")
 #'
 get_mass_props_unc <- function(df, id) {
+  row <- df_get_row_by_id(df, id)
   list(
-    sigma_mass = df_get_by_id(df, id, "sigma_mass"),
-    sigma_center_mass = sapply(c(x = "sigma_Cx", y = "sigma_Cy", z = "sigma_Cz"), FUN=function(p) df_get_by_id(df, id, p)),
+    sigma_mass = row$sigma_mass,
+
+    sigma_center_mass = c(x = row$sigma_Cx, y = row$sigma_Cy, z = row$sigma_Cz),
+
     sigma_inertia = {
       xyz <- list("x", "y", "z")
       sit <- matrix(data = rep.int(0, 9), nrow = 3, dimnames = list(xyz, xyz))
-      sit["x", "x"] <- df_get_by_id(df, id, "sigma_Ixx")
-      sit["y", "y"] <- df_get_by_id(df, id, "sigma_Iyy")
-      sit["z", "z"] <- df_get_by_id(df, id, "sigma_Izz")
-      sit["x", "y"] <- sit["y", "x"] <- df_get_by_id(df, id, "sigma_Ixy")
-      sit["x", "z"] <- sit["z", "x"] <- df_get_by_id(df, id, "sigma_Ixz")
-      sit["y", "z"] <- sit["z", "y"] <- df_get_by_id(df, id, "sigma_Iyz")
+      sit["x", "x"] <- row$sigma_Ixx
+      sit["y", "y"] <- row$sigma_Iyy
+      sit["z", "z"] <- row$sigma_Izz
+      sit["x", "y"] <- sit["y", "x"] <- row$sigma_Ixy
+      sit["x", "z"] <- sit["z", "x"] <- row$sigma_Ixz
+      sit["y", "z"] <- sit["z", "y"] <- row$sigma_Iyz
       sit
     }
   )
@@ -137,7 +144,8 @@ get_mass_props_and_unc <- function(df, id) {
 #' set_mass_props(df, "C.1.2.2.3.2.1.1", mp)
 #'
 set_mass_props <- function(df, id, mp) {
-  m <- mp$inertia
+  cm <- mp$center_mass
+  it <- (mp$inertia + t(mp$inertia)) / 2
   tryCatch(
     poi_factor <- if (mp$poi_conv == "-") 1 else if (mp$poi_conv == "+") -1 else stop(),
     error = function(e) stop("invalid sign convention")
@@ -145,26 +153,22 @@ set_mass_props <- function(df, id, mp) {
   values <- list(
     mass = mp$mass,
 
-    Cx = mp$center_mass["x"],
-    Cy = mp$center_mass["y"],
-    Cz = mp$center_mass["z"],
+    Cx = cm["x"],
+    Cy = cm["y"],
+    Cz = cm["z"],
 
-    Ixx = m["x", "x"],
-    Iyy = m["y", "y"],
-    Izz = m["z", "z"],
+    Ixx = it["x", "x"],
+    Iyy = it["y", "y"],
+    Izz = it["z", "z"],
 
-    Ixy = poi_factor * (m["x", "y"] + m["y", "x"]) / 2.0,
-    Ixz = poi_factor * (m["x", "z"] + m["z", "x"]) / 2.0,
-    Iyz = poi_factor * (m["y", "z"] + m["z", "y"]) / 2.0,
+    Ixy = poi_factor * it["x", "y"],
+    Ixz = poi_factor * it["x", "z"],
+    Iyz = poi_factor * it["y", "z"],
 
     POIconv = mp$poi_conv,
     Ipoint = mp$point
   )
-  Reduce(
-    f = function(d, n) df_set_by_id(d, id, n, values[[n]]),
-    x = names(values),
-    init = df
-  )
+  df_set_row_by_id(df, id, values)
 }
 
 #' Set mass properties uncertainties for a row in a data frame
@@ -187,26 +191,24 @@ set_mass_props <- function(df, id, mp) {
 #' set_mass_props_unc(sawe_table, "Combined", get_mass_props_unc(sawe_table, "Widget"))
 #'
 set_mass_props_unc <- function(df, id, mpu) {
+  sigma_cm <- mpu$sigma_center_mass
+  sigma_it <- (mpu$sigma_inertia + mpu$sigma_inertia) / 2
   values <- list(
     sigma_mass = mpu$sigma_mass,
 
-    sigma_Cx = mpu$sigma_center_mass["x"],
-    sigma_Cy = mpu$sigma_center_mass["y"],
-    sigma_Cz = mpu$sigma_center_mass["z"],
+    sigma_Cx = sigma_cm["x"],
+    sigma_Cy = sigma_cm["y"],
+    sigma_Cz = sigma_cm["z"],
 
-    sigma_Ixx = mpu$sigma_inertia["x", "x"],
-    sigma_Iyy = mpu$sigma_inertia["y", "y"],
-    sigma_Izz = mpu$sigma_inertia["z", "z"],
+    sigma_Ixx = sigma_it["x", "x"],
+    sigma_Iyy = sigma_it["y", "y"],
+    sigma_Izz = sigma_it["z", "z"],
 
-    sigma_Ixy = (mpu$sigma_inertia["x", "y"] + mpu$sigma_inertia["y", "x"]) / 2.0,
-    sigma_Ixz = (mpu$sigma_inertia["x", "z"] + mpu$sigma_inertia["z", "x"]) / 2.0,
-    sigma_Iyz = (mpu$sigma_inertia["y", "z"] + mpu$sigma_inertia["z", "y"]) / 2.0
-  )
-  Reduce(
-    f = function(d, n) df_set_by_id(d, id, n, values[[n]]),
-    x = names(values),
-    init = df
+    sigma_Ixy = sigma_it["x", "y"],
+    sigma_Ixz = sigma_it["x", "z"],
+    sigma_Iyz = sigma_it["y", "z"]
   )
+  df_set_row_by_id(df, id, values)
 }
 
 #' Set mass properties and uncertainties for a row in a data frame
